//
// Licensed to the Apache Software Foundation (ASF) under one or more
// contributor license agreements.  See the NOTICE file distributed with
// this work for additional information regarding copyright ownership.
// The ASF licenses this file to You under the Apache License, Version 2.0
// (the "License"); you may not use this file except in compliance with
// the License.  You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

// .NET Service Configuration V2.0 for Design Considerations for Service-Oriented Applications based on Windows Communication Foundation. Created with Microsoft .NET Framework 3.5 and Microsoft Visual Studio. Copyright 2008, Microsoft Corporation.
//======================================================================================================
// Code originally contributed by Microsoft Corporation.
// This contribution to the Stonehenge project is limited strictly 
// to the source code that is submitted in this submission.  
// Any technology, including underlying platform technology, 
// that is referenced or required by the submitted source code 
// is not a part of the contribution.  
// For example and not by way of limitation, 
// any systems/Windows libraries (WPF, WCF, ASP.NET etc.) 
// required to run the submitted source code is not a part of the contribution
//======================================================================================================
using System;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Runtime.Serialization;
using System.Collections.Generic;
using System.Threading;
using System.Configuration;
using Trade.BusinessServiceContract;
using Trade.BusinessServiceDataContract;
using Trade.StockTraderWebApplicationSettings;
using Trade.Client;
using Trade.Utility;
using Trade.ConfigClient;
using Trade.ConfigServiceContract;
using Trade.ConfigServiceDataContract;


namespace Trade.BusinessServiceClient
{
    /// <summary>
    /// This is the WCF client class for the remote Business Services, connecting to any Web Service platform
    /// via WCF (.NET, J2EE etc.) that implements ITradeServices. This class implements channel initialization and
    /// load balancing/failover logic across cached channel instances specific to the Configuration Management/Node services
    /// implemented in StockTrader via the LoadBalancingClient.Client class, now re-used across all clients 
    /// implementing the configuration service. 
    /// </summary>
    [System.Runtime.Serialization.KnownType(typeof(AccountDataModel))]
    public class BusinessServiceClient : ITradeServices
    {
        public Client.Client<ITradeServices> bslclient;

        public BusinessServiceClient()
        {
            //Clears the cache of the client so that changes in the database configuration can
            //be recognized without restarts
            Initialize();
          
            //Using the database configuration system - this BusinessServiceClient is created based on the CLIENTTOBS
            //Configuration Settings specified in the database i.e. the row containing DOTNET_CLIENT
            var configClient = new ConfigServiceClient();
            var clientRequest = new ClientConfigRequest();
            clientRequest.ClientName = Settings.CLIENT_LABEL;
            var clientConfig = configClient.GetClientConfig(clientRequest);

            //In case the client cannot be properly configured from the database, look up the default ACCESSMODE from the 
            //App.config file
            if (clientConfig == null)
            {
                throw new Exception(StockTraderUtility.EXCEPTION_MESSAGE_BAD_CONFIG);
            }

            //If configuration information was properly received from the database then choose the appropriate business
            //service based on the info in the CLIENTTOBS table i.e. the row with DOTNET_CLIENT
            try
            {
                if (ConfigurationManager.AppSettings.Get(clientConfig.BSName) == null)
                    throw new Exception(string.Format("The WCF endpoint for {0} was not defined in the configuration", clientConfig.BSName));

                bslclient = new Client.Client<ITradeServices>(ConfigurationManager.AppSettings.Get(clientConfig.BSName), clientConfig.BS, Settings.BSL_USERID, Settings.BSL_PASSWORD, true);
            }
            catch
            {
                throw new Exception(string.Format(StockTraderUtility.EXCEPTION_MESSAGE_BAD_CONFIG, clientRequest.ClientName));
            }             
        }


        public ITradeServices Channel
        {
            get
            {
                return bslclient.Channel;
            }
            set
            {
                bslclient.Channel = value;
            }
        }

        public void emptyMethodAction()
        {
            try
            {
                this.Channel.emptyMethodAction();
            }
            catch 
            {
                this.Channel = null;
                throw;                       
            }
        }

        /// <summary>
        /// Logs user in/authenticates against StockTrader database.
        /// </summary>
        /// <param name="userid">User id to authenticate.</param>
        /// <param name="password">Password for authentication</param>
        public AccountDataModel login(string userID, string password)
        {
            try
            {
                if (userID == null)
                    userID = GetDefaultUserId();

                return this.Channel.login(userID, password);
            }
            catch 
            {
                this.Channel = null;
                throw;
            }
        }

        /// <summary>
        /// Gets recent orders for a user.  Transforms data from DataContract to model UI class for HTML display.
        /// </summary>
        /// <param name="userID">User id to retrieve data for.</param>
        public List<OrderDataModel> getOrders(string userID)
        {
            try
            {
                if (userID == null)
                    userID = GetDefaultUserId();
                
                return this.Channel.getOrders(userID);
            }
            catch 
            {
                this.Channel = null;
                throw;
            }
        }

        /// <summary>
        /// Gets account data for a user.  Transforms data from DataContract to model UI class for HTML display.
        /// </summary>
        /// <param name="userID">User id to retrieve data for.</param>
        public AccountDataModel getAccountData(string userID)
        {
            try
            {
                if (userID == null)
                    userID = GetDefaultUserId();


                return this.Channel.getAccountData(userID);
            }
            catch 
            {
                this.Channel = null;
                throw;
            }
        }

        /// <summary>
        /// Gets account profile data for a user.  Transforms data from DataContract to model UI class for HTML display.
        /// </summary>
        /// <param name="userID">User id to retrieve data for.</param>
        public AccountProfileDataModel getAccountProfileData(string userID)
        {
            try
            {
                if (userID == null)
                    userID = GetDefaultUserId();

                return this.Channel.getAccountProfileData(userID);
            }
            catch
            {
                this.Channel = null;
                throw;
            }
        }

        /// <summary>
        /// Updates account profile data for a user. 
        /// </summary>
        /// <param name="profileData">Profile data model class with updated info.</param>
        public AccountProfileDataModel updateAccountProfile(AccountProfileDataModel profileData)
        {
            try
            {
                
                return this.Channel.updateAccountProfile(profileData);
            }
            catch
            {
                this.Channel = null;
                throw;
            }
        }

        /// <summary>
        /// Logs a user out--updates logout count.
        /// </summary>
        /// <param name="userID">User id to logout.</param>
        public void logout(string userID)
        {
            try
            {
                if (userID == null)
                    userID = GetDefaultUserId();

                this.Channel.logout(userID);
            }
            catch
            {
                this.Channel = null;
                throw;
            }
            return;
        }

        /// <summary>
        /// Performs a stock buy operation.
        /// </summary>
        /// <param name="userID">User id to create/submit order for.</param>
        /// <param name="symbol">Stock symbol to buy.</param>
        /// <param name="quantity">Shares to buy</param>
        public OrderDataModel buy(string userID, string symbol, double quantity, int orderProcessingMode)
        {
            try
            {
                if (userID == null)
                    userID = GetDefaultUserId();

                return this.Channel.buy(userID, symbol, quantity, orderProcessingMode);
            }
            catch
            {
                this.Channel = null;
                throw;
            }
        }

        /// <summary>
        /// Performs a holding sell operation.
        /// </summary>
        /// <param name="userID">User id to create/submit order for.</param>
        /// <param name="holdingID">Holding id to sell off.</param>
        /// <param name="quantity">Shares to sell.</param>
        public OrderDataModel sell(string userID, int holdingID, int orderProcessingMode)
        {
            try
            {
                if (userID == null)
                    userID = GetDefaultUserId();

                return this.Channel.sell(userID, holdingID, orderProcessingMode);
            }
            catch
            {
                this.Channel = null;
                throw;
            }
        }

        /// <summary>
        /// Gets holding data for a user.  Transforms data from DataContract to model UI class for HTML display.
        /// </summary>
        /// <param name="userID">User id to retrieve data for.</param>
        public List<HoldingDataModel> getHoldings(string userID)
        {
            try
            {
                if (userID == null)
                    userID = GetDefaultUserId();

                return this.Channel.getHoldings(userID);
            }
            catch
            {
                this.Channel = null;
                throw;
            }
        }

        /// <summary>
        /// Registers/adds new user to database.
        /// </summary>
        /// <param name="userID">User id for account creation/login purposes as specified by user.</param>
        /// <param name="password">Password as specified by user.</param>
        /// <param name="fullname">Name as specified by user.</param>
        /// <param name="address">Address as specified by user.</param>
        /// <param name="email">Email as specified by user.</param>
        /// <param name="creditcard">Credit card number as specified by user.</param>
        /// <param name="openBalance">Open balance as specified by user. </param>
        public AccountDataModel register(string userID, string password, string fullname, string address, string email, string creditcard, decimal openBalance)
        {
            try
            {
                
                return this.Channel.register(userID, password, fullname, address, email, creditcard, openBalance);
            }
            catch
            {
                this.Channel = null;
                throw;
            }
        }

        /// <summary>
        /// Gets any closed orders for a user--orders that have been processed.  Also updates status to complete.
        /// </summary>
        /// <param name="userID">User id to retrieve data for.</param>
        public List<OrderDataModel> getClosedOrders(string userID)
        {
            try
            {
                if (userID == null)
                    userID = GetDefaultUserId();

                return this.Channel.getClosedOrders(userID);
            }
            catch
            {
                this.Channel = null;
                throw;
            }
        }

        /// <summary>
        /// Gets the current market summary.  This results in an expensive DB query in the DAL; hence look to cache data returned for 60 second or so.
        /// </summary>
        public MarketSummaryDataModelWS getMarketSummary()
        {
            try
            {
                
                return this.Channel.getMarketSummary();
            }
            catch
            {
                this.Channel = null;
                throw;
            }
        }

        /// <summary>
        /// Gets a single quote based on symbol.
        /// </summary>
        /// <param name="symbol">Symbol to get data for.</param>
        public QuoteDataModel getQuote(string symbol)
        {
            try
            {
                
                return this.Channel.getQuote(symbol);
            }
            catch
            {
                this.Channel = null;
                throw;
            }
        }

        /// <summary>
        /// Gets a holding for a user.  Transforms data from DataContract to model UI class for HTML display.
        /// </summary>
        /// <param name="userID">User id to retrieve data for.</param>
        /// <param name="holdingid">Holding id to retrieve data for.</param>
        public HoldingDataModel getHolding(string userID, int holdingID)
        {
            try
            {
                if (userID == null)
                    userID = GetDefaultUserId();

                return this.Channel.getHolding(userID, holdingID);
            }
            catch
            {
                this.Channel = null;
                throw;
            }
        }

        /// <summary>
        /// Gets specific top n orders for a user.  Transforms data from DataContract to model UI class for HTML display.
        /// </summary>
        /// <param name="userID">User id to retrieve data for.</param>
        public List<OrderDataModel> getTopOrders(string userID)
        {
            try
            {
                if (userID == null)
                    userID = GetDefaultUserId();

                return this.Channel.getTopOrders(userID);
            }
            catch
            {
                this.Channel = null;
                throw;
            }
        }

        /// <summary>
        /// Allows user to sell part of a holding vs. all.  Not implemented by Trade 6.1 on WebSphere.
        /// </summary>
        /// <param name="userID">User id to submit sell for.</param>
        /// <param name="holdingID">Holding id to sell.</param>
        /// <param name="quantity">Number of shares to sell.</param>
        public OrderDataModel sellEnhanced(string userID, int holdingID, double quantity)
        {
            try
            {
                if (userID == null)
                    userID = GetDefaultUserId();
                
                return this.Channel.sellEnhanced(userID, holdingID, quantity);
            }
            catch
            {
                this.Channel = null;
                throw;
            }
        }

        /// <summary>
        /// Simple online check method.
        /// </summary>
        public void isOnline()
        {
            try
            {
                
                this.Channel.isOnline();
                return;
            }
            catch
            {
                this.Channel = null;
                throw;
            }
        }

        /// <summary>
        /// A Static method that clears the cache of the Client Service
        /// </summary>
        public static void Initialize()
        {
            Client.Client<ITradeServices>.ClearCache();
        }

        /// <summary>
        /// TODO: Hack for metro
        /// </summary>
        /// <returns></returns>
        private string GetDefaultUserId()
        {
            return Settings.BSL_USERID;
        }

    }
}